package ovh.toms.flyfeather.util;

import org.bukkit.Bukkit;
import org.bukkit.command.Command;
import org.bukkit.command.CommandExecutor;
import org.bukkit.command.CommandMap;
import org.bukkit.command.CommandSender;
import org.bukkit.event.Listener;
import org.bukkit.plugin.Plugin;

import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.util.Arrays;
import java.util.logging.Level;
import java.util.logging.Logger;


/**
 * Created by Tom S. on 15.12.17
 * This code is property of Tysox.de Development
 * (c) 2015-2017
 */
public class RegisterUtil<P extends Plugin> {

    private static final String VERSION;
    private static CommandMap commandMap = null;

    static{
        String path = Bukkit.getServer().getClass().getPackage().getName();
        VERSION = path.substring(path.lastIndexOf(".") + 1, path.length());
        try {
            Class<?> craftServerClass = Class.forName("org.bukkit.craftbukkit." + VERSION + ".CraftServer");
            Field commandMapField = craftServerClass.getDeclaredField("commandMap");
            commandMapField.setAccessible(true);
            commandMap = (CommandMap) commandMapField.get(Bukkit.getServer());
        } catch (ClassNotFoundException | NoSuchFieldException | IllegalAccessException e) {
            e.printStackTrace();
        }
    }

    private P plugin;

    public RegisterUtil(P plugin) {
        this.plugin = plugin;
    }

    public void registerEvents(Class<?>... listeners) {
        for (Class<?> clazz : listeners) {
            boolean isConstructor;
            try {
                clazz.getConstructor(plugin.getClass());
                isConstructor = true;
            } catch (NoSuchMethodException ex1) {
                isConstructor = false;
            }

            try {
                Listener listener;
                if (isConstructor) {
                    Constructor<?> cww = clazz.getConstructor(plugin.getClass());
                    listener = (Listener) cww.newInstance(plugin);
                } else {
                    listener = (Listener) clazz.newInstance();
                }
                Bukkit.getPluginManager().registerEvents(listener, plugin);
            } catch (Exception ex) {
                Logger.getLogger(RegisterUtil.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }

    public void registerCommand(String name, String description, CommandExecutor commandExecutor, String... aliases) {
        try {
            DynCommand dynCmd = new DynCommand(name, description, commandExecutor, aliases);
            commandMap.register(plugin.getName(), dynCmd);
        } catch (Exception ex) {
            Logger.getLogger(RegisterUtil.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    private class DynCommand extends Command {
        private CommandExecutor exec;

        protected DynCommand(String name, String description, CommandExecutor exec, String... aliases) {
            super(name);
            this.exec = exec;
            super.setDescription(description);
            super.setAliases(Arrays.asList(aliases));
        }

        @Override
        public boolean execute(CommandSender cs, String label, String[] args) {
            exec.onCommand(cs, this, label, args);
            return false;
        }
    }

}
