package ovh.toms.flyfeather.util;

import com.google.common.base.Charsets;
import com.google.common.io.Files;
import lombok.Getter;
import org.apache.commons.lang.Validate;
import org.bukkit.configuration.file.YamlConfiguration;

import java.io.*;

@Getter
public abstract class SimpleConfig {

	private File file = null;
    private YamlConfiguration configuration = null;

	/**
	 * @param fileName
	 */
	public SimpleConfig(String fileName) {
		this.file = new File("plugins/FlyFeather", fileName + ".yml");
		this.configuration = YamlConfiguration.loadConfiguration(file);
		if (!this.file.exists()) {
			try {
				this.file.createNewFile();
				this.load();
			} catch (IOException e) {
				System.err.println("Es ist ein Fehler beim erstellen der Config aufgetreten");
			}
			this.initConfig();
		}

	}

	public abstract void initConfig();

	public void load() {
		try {
			if (!this.file.exists()) {
				this.file.createNewFile();
				this.initConfig();
			} else {
				this.configuration.load(new InputStreamReader(new FileInputStream(this.file), Charsets.UTF_8));
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public void save() {
		try {
			Validate.notNull(this.file, "File cannot be null");
			Files.createParentDirs(this.file);
			String data = this.configuration.saveToString();
			Writer writer = new OutputStreamWriter(new FileOutputStream(this.file), Charsets.UTF_8);
			try {
				writer.write(data);
			} finally {
				writer.close();
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public void reload() {
		this.save();
		this.load();
	}

	/**
	 * @param path
	 * @return the highest number
	 */
	public int getHighest(String path) {
		int highest = 0;
		try {
			for (String sign : configuration.getConfigurationSection(path).getKeys(false)) {
				int i = Integer.parseInt(sign);

				if (i > highest) {
					highest = i;
				}
			}
		} catch (Exception e) {
			highest = 0;
		}
		return highest;
	}
}